import { h, render } from 'preact';
import { tap, delay } from 'rxjs/operators';
import { createComponent, } from '../component';
import { combine } from '../util/combine';
import { createRef } from '../util/createRef';
describe('Component', () => {
    it('renders a component', () => {
        const cfn = ({ props }) => props;
        const ct = state => h("div", null, state.foo);
        const Component = createComponent(cfn, ct);
        const root = document.createElement('div');
        render(h(Component, { foo: "bar" }), root);
    });
    it('throws when component function does not emit', () => {
        const cfn = () => combine({ foo: 'bar' }).pipe(delay(300));
        const ct = state => h("div", null, state.foo);
        const Component = createComponent(cfn, ct);
        const root = document.createElement('div');
        expect(() => {
            render(h(Component, null), root);
        }).toThrow();
    });
    it('supports refs', done => {
        const cfn = ({ subscribe }) => {
            const [ref, elementStream] = createRef();
            subscribe(elementStream.pipe(tap(el => {
                expect(el).toBeInstanceOf(HTMLDivElement);
                done();
            })));
            return combine({ ref });
        };
        const ct = state => (h("div", { ref: state.ref }));
        const Component = createComponent(cfn, ct);
        const root = document.createElement('div');
        render(h(Component, null), root);
    });
});
