import { Component, Fragment, h } from 'preact';
import { BehaviorSubject, Subject, Subscription } from 'rxjs';
import { distinctUntilChanged } from 'rxjs/operators';
import { shallowEqual } from './util/shallowEqual';
class BassdrumComponent extends Component {
    constructor(props, context) {
        super(props, context);
        this.propsStream = new BehaviorSubject(props);
        this.updates = new Subject();
        this.subscriptions = [];
        this.subscriptions.push(this.createSubscription());
    }
    createSubscription() {
        return new Subscription();
    }
    componentDidMount() {
        this.updates.next(this.props);
    }
    componentWillReceiveProps(props) {
        this.propsStream.next(props);
    }
    shouldComponentUpdate(_, nextState) {
        return this.state !== nextState;
    }
    componentDidUpdate() {
        this.updates.next(this.props);
    }
    render() {
        return h(Fragment, null);
    }
    componentWillUnmount() {
        this.propsStream.complete();
        this.updates.complete();
        this.subscriptions.forEach(sub => sub.unsubscribe());
        this.subscriptions.length = 0;
    }
}
const createComponentAPI = (component) => ({
    props: component.propsStream,
    subscribe: (obs) => component.subscriptions.push(obs.subscribe()),
    updates: component.updates,
});
export const createComponent = (componentFunction, template) => class extends BassdrumComponent {
    createSubscription() {
        let hasEmitted = false;
        const subscription = componentFunction(createComponentAPI(this))
            .pipe(distinctUntilChanged(shallowEqual))
            .subscribe(state => {
            hasEmitted = true;
            this.setState(state);
        });
        if (!hasEmitted) {
            throw new Error('Your Component did not emit any state when it was created. ' +
                'Make sure the Observable you return from your ' +
                'component function emits immediately.');
        }
        return subscription;
    }
    render() {
        return template(this.state);
    }
};
